@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    mkfs.c -- program for creating reiser4 filesystem. */
@@ -29,15 +29,13 @@
 #include <aux/aux.h>
 #include <misc/misc.h>
 
-enum mkfs_behav_flags {
+typedef enum mkfs_behav_flags {
 	BF_FORCE      = 1 << 0,
 	BF_YES        = 1 << 1,
 	BF_LOST       = 1 << 2,
 	BF_SHOW_PARM  = 1 << 3,
 	BF_SHOW_PLUG  = 1 << 4
-};
-
-typedef enum mkfs_behav_flags mkfs_behav_flags_t;
+} mkfs_behav_flags_t;
 
 /* Prints mkfs options */
 static void mkfs_print_usage(char *name) {
@@ -78,25 +76,22 @@ static void mkfs_init(void) {
 
 static reiser4_object_t *reiser4_root_create(reiser4_fs_t *fs) {
 	entry_hint_t entry;
-	object_hint_t hint;
+	object_info_t info;
 	
 	aal_assert("vpf-1625", fs != NULL);
 	aal_assert("vpf-1626", fs->tree != NULL);
 	
-	aal_memset(&hint, 0, sizeof(hint));
+	aal_memset(&info, 0, sizeof(info));
 
-	/* Preparing object hint. INVAL_PTR means that this kind of plugin 
-	   is not ready yet but needs to be stored, */
-	hint.mode = 0;
-	reiser4_opset_profile(hint.info.opset.plug);
-	hint.info.opset.plug[OPSET_OBJ] = hint.info.opset.plug[OPSET_MKDIR];
-	hint.info.tree = (tree_entity_t *)fs->tree;
+	info.tree = (tree_entity_t *)fs->tree;
 
 	/* Preparing entry hint. */
 	entry.name[0] = '\0';
 	aal_memcpy(&entry.offset, &fs->tree->key, sizeof(entry.offset));
 
-	return reiser4_object_create(&entry, &hint);
+	reiser4_pset_root(&info);
+	
+	return reiser4_object_create(&entry, &info, NULL);
 }
 
 int main(int argc, char *argv[]) {
@@ -221,11 +216,6 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if (optind >= argc) {
-		mkfs_print_usage(argv[0]);
-		return USER_ERROR;
-	}
-    
 	if (!(flags & BF_YES))
 		misc_print_banner(argv[0]);
 
@@ -258,7 +248,13 @@ int main(int argc, char *argv[]) {
 
 	if (optind >= argc)
 		goto error_free_libreiser4;
-		
+	
+	if (optind >= argc) {
+		mkfs_print_usage(argv[0]);
+		return USER_ERROR;
+	}
+    
+
 #ifdef HAVE_SYSCONF
 	/* Guessing block size by getting page size */
 	if (!hint.blksize) {
@@ -304,15 +300,15 @@ int main(int argc, char *argv[]) {
 
 	if (!(flags & BF_FORCE)) {
 		if (aal_strncmp(sysinfo.release, "2.5", 3) &&
-		    aal_strncmp(sysinfo.release, "2.6", 3))
-		{
+		    aal_strncmp(sysinfo.release, "2.6", 3) &&
+		    (aal_strncmp(sysinfo.release, "3.", 2) ||
+		     (sysinfo.release[2] < '0' && (sysinfo.release[2] > '9')))){
 			aal_warn("%s %s is detected. Reiser4 does not "
 				 "support such a platform. Use -f to "
 				 "force over.", sysinfo.sysname,
 				 sysinfo.release);
 			goto error_free_libreiser4;
 		}
-
 	}
 
 	if (!(flags & BF_YES)) {
@@ -476,17 +472,23 @@ int main(int argc, char *argv[]) {
 			goto error_free_fs;
 		}
 	
-		if (reiser4_opset_init(fs->tree, 1)) {
+		if (reiser4_pset_tree(fs->tree, 1)) {
 			aal_error("Can't initialize the fs-global "
 				  "object plugin set.");
 			goto error_free_fs;
 		}
 
+		/* Backup the fs metadata. */
+		if (!(fs->backup = reiser4_backup_create(fs))) {
+			aal_error("Can't create the fs metadata backup.");
+			goto error_free_fs;
+		}
+
 		/* Creating lost+found directory */
 		if (flags & BF_LOST) {
 			reiser4_object_t *object;
 	    
-			if (!(object = reiser4_dir_create(fs, fs->root,
+			if (!(object = reiser4_dir_create(fs->root,
 							  "lost+found")))
 			{
 				aal_error("Can't create \"/lost+found\" "
